#!/usr/bin/env python
#
""" Give here the function description(s) """

import string
import re

underscore = re.compile(r'(?P<lead>.*?[^\\])(?P<text>[_#$&])(?P<trail>.*)')

def escape_underscore( string ):
    str_out = string
    
    match = underscore.match( str_out )
    while match:
        str_out = r'%(lead)s\%(text)s%(trail)s' % match.groupdict()
        match = underscore.match( str_out )
    return str_out

def show_union_in_struct( fp ):
    found_struct_name = False
    found_ifdef = False
    found_else = False
    while 1:
        line = fp.readline()
        if not line: break

        words = line.split()
        if len(words) > 0:
            if words[0] == 'struct':
                struct_cbuff = []
                continue

            if words[0] == '}':
                if not found_struct_name:
                    struct_name = escape_underscore(words[1][0:-1])
                    found_struct_name = True
                    continue
                else:
                    union_name = escape_underscore(words[1][0:-1])
                    break

            if not found_ifdef and words[0] == '#ifdef':
                found_ifdef = True
            elif found_ifdef:
                if words[0] == '#endif':
                    found_ifdef = False
                    found_else = False
                elif words[0] == '#else':
                    found_else = True
            
            pos = line.find( ';' )
            if pos > 0:
                line = line[0:pos]
                words = line.split()
                pos = words[-1].find( '[' )
                if pos > 0:
                    cbuff = '%s &' % escape_underscore(words[-1][0:pos])
                    pos1 = pos+1
                    pos2 = words[-1].find(']')
                    elements = escape_underscore(words[-1][pos1:pos2])
                else:
                    cbuff = '%s &' % escape_underscore(words[-1])
                    elements = '1'
                for word in words[:-1]:
                    cbuff += ' %s' %  escape_underscore( word )
                cbuff += ' & %s \\\\' % elements
                if not found_else: struct_cbuff.append(cbuff)

    for cbuff in struct_cbuff[0:-1]:
        print '%s.%s.%s' % (union_name, struct_name, cbuff)
    print '%s.%s' % (union_name, struct_cbuff[-1])
                
def translate( fp ):
    while 1:
        line = fp.readline()
        if not line: break

        words = line.split()
        if len(words) == 0: continue
        if words[0] == 'struct':
            print '\\subsection{struct %s}\\label{struct:%s}' % \
                  (escape_underscore(words[1]), words[1])
            print '\\begin{longtable}[l]{l|l|r}'
            print 'Attribute & Data Type & Vector Length \\\\'
            print '\\hline'
            print '\\endfirsthead'
            print 'Attribute & Data Type & Vector Length \\\\'
            print '\\hline'
            print '\\endhead'
            
            while 1:
                line = fp.readline()
                if not line: return
                
                if len(line) > 1 and line[0:2] == '};':
                    print '\\end{longtable}\n'
                    break

                pos = line.find( ';' )
                if pos >= 0:
                    line = line[0:pos]
                    words = line.split()
                    pos = words[-1].find( '[' )
                    if pos > 0:
                        cbuff = '%s &' % escape_underscore(words[-1][0:pos])
                        pos1 = pos+1
                        pos2 = words[-1].find(']')
                        elements = escape_underscore(words[-1][pos1:pos2])
                    else:
                        cbuff = '%s &' % escape_underscore(words[-1])
                        elements = '1'
                    for word in words[:-1]:
                        cbuff += ' %s' %  escape_underscore( word )

                    if words[0] == 'struct':
                        cbuff += ' (\\ref{struct:%s})' % words[1]
                    elif words[0] == 'union':
                        cbuff += ' (\\ref{union:%s})' % words[1]
                    print '%s & %s \\\\' % (cbuff, elements)
                elif len(line) >= 5:
                    words = line.split()
                    if words[0] == 'union':
                        show_union_in_struct( fp )
        elif words[0] == 'union':
            union_name = escape_underscore(words[1])
            print '\\subsection{union %s}\\label{union:%s}' % \
                  (union_name, words[1])
            print '\\begin{tabular}[l]{l|l|r}'
            print 'Attribute & Data Type & Vector Length \\\\'
            print '\\hline'

            while 1:
                found_ifdef = False
                found_else = False
                line = fp.readline()
                if not line: return
                
                if len(line) > 1 and line[0:2] == '};':
                    for cbuff in struct_cbuff[0:-1]:
                        print '%s.%s' % (struct_name, cbuff)
                    print struct_cbuff[-1]
                    print '\\end{tabular}\n'
                    break

                words = line.split()
                if len(words) > 0:
                    if words[0] == 'struct':
                        struct_cbuff = []
                        continue

                    if words[0] == '}':
                        struct_name = escape_underscore(words[1][0:-1])
                        continue

                    if not found_ifdef and words[0] == '#ifdef':
                        found_ifdef = True
                    elif found_ifdef:
                        if words[0] == '#endif':
                            found_ifdef = False
                            found_else = False
                        elif words[0] == '#else':
                            found_else = True
            
                    pos = line.find( ';' )
                    if pos > 0:
                        line = line[0:pos]
                        words = line.split()
                        pos = words[-1].find( '[' )
                        if pos > 0:
                            cbuff = '%s &' % escape_underscore(words[-1][0:pos])
                            pos1 = pos+1
                            pos2 = words[-1].find(']')
                            elements = escape_underscore(words[-1][pos1:pos2])
                        else:
                            cbuff = '%s &' % escape_underscore(words[-1])
                            elements = '1'
                        for word in words[:-1]:
                            cbuff += ' %s' %  escape_underscore( word )
                        cbuff += ' & %s \\\\' % elements
                        if not found_else: struct_cbuff.append(cbuff)

if __name__ == '__main__':
    from optparse import OptionParser

    usage = "usage: %prog [options]"
    parser = OptionParser(usage)
    parser.add_option("--includefile", dest="includefile", type='string')
    parser.set_defaults( directory='Include file' )
    (options, args) = parser.parse_args()

    fp = open( args[0], 'r' )
    translate( fp )
    fp.close()
