#
# COPYRIGHT (c) 2002 - 2013 SRON (R.M.van.Hees@sron.nl)
#
#   This is free software
#   you can redistribute it and/or modify it under the terms of the 
#   GNU General Public License, version 2, as published by the 
#   Free Software Foundation.
#
#   The software is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY# without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program# if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, 
#   Boston, MA  02111-1307, USA.
#
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# 
# Default rules for make to compile C, C++ and Fortran programs
# Tested on HP-UX	(native compiler) 
#           SGI-IRIX	(native compiler (32bits))
#           Linux/GNU	(gcc, intel and pgi)
# 
# You may use the following variables to override the choices made by this
# script, or to help it to find libraries and programs with nonstandard 
# names/locations on your unsupported (?) system. Recognised variables are:
#
#  PREFIX	:	install NADC software in PREFIX
#
#  WITH_CC	:	native/gnu/pgi/intel
#  CFLAGS	:	C compiler flags
#  CXXFLAGS	:	C++ compiler flags
#  FFLAGS	:	Fortran compiler flags
#  LDFLAGS	:	linker flags, e.g. -L<lib dir> if you have libraries 
#			in a nonstandard directory <lib dir>
#
#  WITH_SHARED=0/1 :    compile using shared libraries (=1), or static
#                       libraries (=0)
#
#  WITH_PSQL=0/1   :	compile including PostgreSQL support (=1), or without
#
#  HDF5_DIR=<dir>  :	prefix where HDF5-library is installed (optional)
#
#  WITH_IDL=0/1    :	compile including IDL support (=1), or without (=0)
#  IDL_DIR=<dir>   :	prefix where IDL-software is installed (optional)
#
#  WITH_PPF=0/1    :    compile including PPF support (=1), or without (=0)
#  PPF_DIR=<dir>   :    prefix where PPF-software is installed (optional)
#
#+++++++++++++++++++++++++ History +++++++++++++++++++++++++
# Richard van Hees, 11 March 1997
# RvH, 7 October 1998:	ported to HP-UX
# RvH, 30 Juli 1999:	reorganized file
# RvH, 10 Feb 2000:	ported to SGI-IRIX
# RvH, 29 Mar 2000:	added WWWHOST: hostname to store WWW documentation
# RvH, 16 May 2001:	rearranged directories to create RPM
# RvH, 13 Jul 2001:	cleanup and reorganisation
# RvH, 31 Aug 2001:     ported for the PGI compiler
# RvH, 03 Aug 2001:	made libraries optional, more compiler tuning
# RvH, 10 Feb 2002:     ported to Sun Solaris
# RvH, 15 Oct 2002:     added path to IDL-programs: IDLDIR
# RvH, 15 Oct 2002:     added path to SRON calibration parameters: DATADIR
# RvH, 16 Sep 2003:     reorganized files, added new options
# RvH, 08 Nov 2004:     added search script for hdf4 and hdf5(_hl)
# RvH, 17 Nov 2005:	bug-fix - handle presence of HDF4 library
# RvH, 20 Jun 2008:     remove HDF4 support
#
#++++++++++++++++++	OSTYPE
# Linux/GNU:		Linux
# HP-UX:		HP-UX
# Sun (Solaris):	SunOS
# SGI-IRIX:		IRIX64
#
# defining or overwriting environment variables
SHELL	= /bin/sh
OSTYPE	= $(shell uname)

#
find_h5 = $(shell if [ -r /usr/local/lib/libhdf5.settings ]; \
                  then echo "/usr/local"; else echo "/usr"; fi;)
find_ppf = $(shell if [ -r /usr/local/include/ppf_lib.h ]; \
		   then echo "/usr/local"; else echo "$(NADC_EXTERN)"; fi;)

#
# default directories to install libraries and programs
ifeq ($(shell uname -m),x86_64)
PREFIX	:= $(HOME)/x86_64
else
PREFIX	:= $(HOME)
endif

# default location for the binaries
BINDIR	:= $(PREFIX)/bin

# default location for the libraries
LIBDIR	:= $(PREFIX)/lib

# default location for documentation
DOCDIR	:= $(PREFIX)/share/doc

# default location for the IDL library
IDLDIR	:= $(PREFIX)/share/nadc_idl

# location for the Auxiliary data (at SRON)
DATADIR = /SCIA/share/nadc_tools

# machine to put the Web-pages on
WWWHOST	= lepus
WWWDIR	= public_html/SciaDC

# default is to compile with shared libraries
ifndef WITH_SHARED
	WITH_SHARED = 1
endif

# default is to include PostgreSQL support
ifndef WITH_PSQL
#	PSQL_KNMI = 1    # uncomment to compile old PSQL code used at KNMI
	WITH_PSQL = 1
endif

# default is to include PPF support
ifndef WITH_PPF
	WITH_PPF = 1
endif

# default is to include IDL support
ifndef WITH_IDL
	WITH_IDL = 1
endif

#+++++++++++++++++++++++++ required definitions (do not edit!)
# default place for the include files of this package
LIBPATH	:= $(TOPDIR)/.libs
INCPATH	:= $(TOPDIR)/include
ifndef ADD_INC
	ADD_INC	= 
endif
ADD_CFLAGS =
ifndef SUBLIBDIR
	SUBLIBDIR =
endif

#+++++++++++++++++++++++++ location of HDF5 - IDL
SZ_DIR = /usr

# where are the HDF5 include files and libraries?
ifndef HDF5_DIR
	HDF5_DIR := $(call find_h5)
	ifeq ($(OSTYPE),HP-UX)
		HDF5_DIR := /usr/local/hdf5
	endif
	ifeq ($(OSTYPE),SunOS)
		HDF5_DIR := $(HOME)/hdf5
	endif
	ifeq ($(OSTYPE),IRIX64)
		HDF5_DIR := $(HOME)/hdf5
		SZ_DIR := $(HOME)/szip
	endif
endif

# where are the IDL include files and libraries?
ifndef IDL_DIR
	IDL_DIR = /usr/local/itt/idl
	ifeq ($(OSTYPE),HP-UX)
		IDL_DIR = /usr/local/rsi/idl
	endif
	ifeq ($(OSTYPE),SunOS)
		IDL_DIR = /opt/rsi/idl
	endif
endif

# where are the PPF include files and libraries?
ifndef PPF_DIR
	PPF_DIR := $(call find_ppf)
endif

#+++++++++++++++++++++++++ Libraries
HDF5_INC =
HDF5_LIB =
SZ_LIB =
ifneq ($(shell [ -r $(HDF5_DIR)/include/hdf5.h ] && echo yes),yes)
	$(error nadc_tools requires HDF5 version 1.8.x)
endif
_WITH_NC4 := $(shell if type nc-config > /dev/null 2>&1; then \
			nc-config --has-nc4; else echo "no"; fi)

SQL_INC = 
SQL_LIB = 
ifeq ($(WITH_PSQL),1)
	_WITH_SQL := $(shell if type pg_config > /dev/null 2>&1; then \
			echo "yes"; else echo "no"; fi)
else
	_WITH_SQL = no
endif

PPF_INC =
PPF_LIB =
ifeq ($(WITH_PPF),1)
	_WITH_PPF := $(shell [ -r $(PPF_DIR)/include/ppf_lib.h ] && echo yes)
else
	_WITH_PPF = no
endif

IDL_INC =
ifeq ($(WITH_IDL),1)
	_WITH_IDL := $(shell [ -r $(IDL_DIR)/external/export.h ] && echo yes)
	IDL_INC := -I$(IDL_DIR)/external
else
	_WITH_IDL = no
endif

_WITH_PATCH = no
_WITH_PATCH := $(shell [ -d $(TOPDIR)/PatchScia ] && echo yes)

MLIB	= -lm
#+++++++++++++++++++++++++ Compiler settings
ifndef WITH_CC
	ifeq ($(OSTYPE),HP-UX)
		include $(TOPDIR)/config/config-hpux
	endif
	ifeq ($(OSTYPE),IRIX64)
		include $(TOPDIR)/config/config-irix
	endif
	ifeq ($(OSTYPE),SunOS)
		include $(TOPDIR)/config/config-sunos
	endif
	ifeq ($(OSTYPE),Linux)
		include $(TOPDIR)/config/config-gcc
	endif
else
	ifeq ($(WITH_CC),gnu)
		include $(TOPDIR)/config/config-gcc
	endif
	ifeq ($(WITH_CC),pgi)
		include $(TOPDIR)/config/config-pgi
	endif
	ifeq ($(WITH_CC),intel)
		include $(TOPDIR)/config/config-intel
	endif
endif
ifdef PG_OPT
	CFLAGS	+= $(PG_OPT)
	CXXFLAGS += $(PG_OPT)
	FFLAGS	+= $(PG_OPT)
	LDFLAGS	+= $(PG_OPT)
endif

#+++++++++++++++++++++++++ add CPP flags
HDF5_INC := -I$(HDF5_DIR)/include
HDF5_LIB := -L$(HDF5_DIR)/lib -lhdf5_hl -lhdf5 -ldl
ifeq ($(shell [ -r $(SZ_DIR)/include/szlib.h ] && echo yes),yes)
	SZ_LIB := -L$(SZ_DIR)/lib -lsz
endif
ifeq ($(_WITH_NC4),yes)
	CFLAGS += -D_WITH_NC4
	NC4_LIB := `nc-config --libs`
endif

ifeq ($(_WITH_SQL),yes)
	CFLAGS += -D_WITH_SQL
ifeq ($(PSQL_KNMI),1)
	CFLAGS += -D_KNMI_SQL
endif
	SQL_INC := -I`pg_config --includedir`
	SQL_LIB := -L`pg_config --libdir` -lpq
endif
ifeq ($(_WITH_PPF),yes)
	PPF_INC := -I$(PPF_DIR)/include
	PPF_LIB := -L$(PPF_DIR)/lib -lppf_pointing -lppf_orbit -lppf_lib
endif

#+++++++++++++++++++++++++ Shared library rules
ifeq ($(SHARED_LIBS),yes)
	ifeq ($(OSTYPE),HP-UX)
		SO_EXT	= sl
		LDFLAGS += -L$(LIBDIR)
	else
		SO_EXT	= so
	endif
else
	ifeq ($(OSTYPE),HP-UX)
		LDFLAGS += -Wl,-a,archive_shared -L$(LIBDIR)
	endif
endif

#+++++++++++++++++++++++++ Static library rules
AR	= ar
AR_OPT	= ruvs
AR_LIB	= $(LIBNAME).a

#+++++++++++++++++++++++++ Miscellaneous
INSTALL = install -m 0755
INSTALL_RD = install -m 0644

ifeq ($(OSTYPE),Linux)
	MAKE = make
else
	MAKE := $(shell if type gmake > /dev/null 2>1; \
                 then echo "gmake"; else echo "make"; fi;)
endif

STRIP	= strip
RM	= rm -f
LN	= ln -s -f
MKDIR	:= $(TOPDIR)/scripts/mkinstalldirs

_DEFINED_NADC_RULES = yes
