/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.COPYRIGHT (c) 1999 - 2013 SRON (R.M.van.Hees@sron.nl)

   This is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License, version 2, as
   published by the Free Software Foundation.

   The software is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, 
   Boston, MA  02111-1307, USA.

.IDENTifer   defs_GOME
.AUTHOR      R.M. van Hees 
.KEYWORDS    header file
.LANGUAGE    ANSI-C
.PURPOSE     macros and structures for GOME level 1b and 2 data modules
.ENVIRONment none
.VERSION      3.0   08-Jun-2009 update to product version 2, RvH
              2.2   31-Mar-2003	modified include for C++ code
              2.1   06-Mar-2003	added global "Use_Extern_Alloc", RvH 
              2.0   21-Aug-2001 combines level 1b and 2 definitions, RvH
              1.1   12-Jul-2001 added a few handy enumerations, RvH
              1.0   02-Feb-1999 created by R.M. van Hees
------------------------------------------------------------*/

#ifndef  __DEFS_GOME                            /* Avoid redefinitions */
#define  __DEFS_GOME

#ifdef __cplusplus
extern "C" {
#endif

/*+++++ Macros +++++*/
#define GOME_CHAR   ((size_t) 1)
#define GOME_SHORT  ((size_t) 2)
#define GOME_USHRT  ((size_t) 2)
#define GOME_INT    ((size_t) 4)
#define GOME_UINT   ((size_t) 4)
#define GOME_FLOAT  ((size_t) 4)
#define GOME_DBLE   ((size_t) 8)

#define LVL1_PIR_LENGTH           38
#define LVL2_FSR_LENGTH           12
#define LVL1_FSR_LENGTH           96
#define LVL2_SPH_LENGTH          127
#define LVL2_GLR_LENGTH          136
#define LVL2_IRR_STATIC_LENGTH   142
#define LVL2_DDR_STATIC_LENGTH     8
#define LVL2_DDR_NWIN_LENGTH      28
#define LVL2_DDR_NMOL_LENGTH      24

#define LVL2_MAX_NWIN              5
#define LVL2_MAX_NMOL             10

#define LVL2_V2_NWIN               2
#define LVL2_V2_NMOL               2

#define MPH_SIZE                  34
#define SPH_SIZE                  22
#define IHR_SIZE                 198

#define PRE_DISP_TEMP_ENTRY      148       /* prism temp entry point in IHR */

#define NUM_STRAY_GHOSTS           2
#define NUM_SPEC_COEFFS            5
#define NUM_POLAR_COEFFS  ((short) 8)
#define NUM_FPA_SCALE              5
#define NUM_FPA_COEFFS           100

#define NUM_BAND_IN_CHANNEL        2
#define PMD_NUMBER                 3
#define PMD_IN_GRID               16
#define NUM_SPEC_BANDS            10
#define SCIENCE_CHANNELS           4
#define CHANNEL_SIZE            1024
#define SCIENCE_PIXELS      (SCIENCE_CHANNELS * CHANNEL_SIZE)

#define FLAG_UNSET          ((unsigned char) 0)
#define FLAG_EARTH          ((unsigned char) 1)
#define FLAG_MOON           ((unsigned char) 2)
#define FLAG_SUN            ((unsigned char) 3)

/*+++++ Global Variables +++++*/
/* 
 * Most routines to read SCIAMACHY data can allocate memory internally
 * However IDL requires the use of their own memory allocation routines
 */
extern bool Use_Extern_Alloc;

/*+++++ Enumarations +++++*/
enum polar_coeffs { PC_PMD_1 = 0,
		    PC_PMD_2,
		    PC_PMD_3,
		    PC_OVERLAP_1,
		    PC_OVERLAP_2,
		    PC_OVERLAP_3,
		    PC_P7,
		    PC_P7_BAND_1A 
};

enum gome_bands { BAND_1a = 0, 
		  BAND_1b, 
		  BAND_2a, 
		  BAND_2b, 
		  BAND_3, 
		  BAND_4,
		  BLIND_1a, 
		  STRAY_1a, 
		  STRAY_1b, 
		  STRAY_2a 
};

/*+++++ Structures & Unions +++++*/
struct mjd_gome
{
     unsigned int days;   /* UTC days since 1.1.1950 */
     unsigned int msec;   /* UTC ms since midnight */
};

struct pir_gome
{
     char mission[3];
     char sensor[4];
     char orbit[6];
     char nr_orbit[5];
     char acquis[3];
     char product[6];
     char blank[2];
     char proc_id[3];
     char proc_date[9];
     char proc_time[7];
};

struct fsr1_gome
{
     short nr_sph;
     int   sz_sph;
     short nr_fcd;
     int   sz_fcd;
     short nr_pcd;
     int   sz_pcd;
     short nr_scd;
     int   sz_scd;
     short nr_mcd;
     int   sz_mcd;
     short nr_band[NUM_SPEC_BANDS];
     int   sz_band[NUM_SPEC_BANDS];
};

struct fsr2_gome
{
     short nr_sph;
     int   sz_sph;
     short nr_ddr;
     int   sz_ddr;
};

struct sph1_gome
{
     short nr_inref;
     char  inref[2][39];
     char  soft_version[6];
     char  calib_version[6];
     short prod_version;
     unsigned int time_orbit;
     unsigned int time_utc_day;
     unsigned int time_utc_ms;
     unsigned int time_counter;
     unsigned int time_period;
     short pmd_entry;
     short subset_entry;
     short intgstat_entry;
     short peltier_entry;
     short status2_entry;
     float pmd_conv[2 * PMD_NUMBER];
     unsigned int state_utc_day;
     unsigned int state_utc_ms;
     unsigned int state_orbit;
     float  state_x;
     float  state_y;
     float  state_z;
     float  state_dx;
     float  state_dy;
     float  state_dz;
     double att_yaw;
     double att_pitch;
     double att_roll;
     double att_dyaw;
     double att_dpitch;
     double att_droll;
     int    att_flag;
     int    att_stat;
     double julian;
     double semi_major;
     double excen;
     double incl;
     double right_asc;
     double perigee;
     double mn_anom;
     struct mjd_gome start_time;
     struct mjd_gome stop_time;
};

struct sph2_gome
{
     char  inref[39];
     char  soft_version[6];
     char  param_version[6];
     char  format_version[6];
     short nwin;
     short nmol;
     float height;
     float window[LVL2_MAX_NWIN];
     short mol_win[LVL2_MAX_NMOL];
     char  mol_name[LVL2_MAX_NMOL][6];
     struct mjd_gome start_time;
     struct mjd_gome stop_time;
};

struct glr1_gome
{
     char  sun_glint;
     unsigned char subsetcounter;
     unsigned int utc_date;
     unsigned int utc_time;
     float sat_geo_height;
     float earth_radius;
     float sun_zen_sat_north[3];
     float sun_azim_sat_north[3];
     float los_zen_sat_north[3];
     float los_azim_sat_north[3];
     float sun_zen_sat_ers[3];
     float sun_azim_sat_ers[3];
     float los_zen_sat_ers[3];
     float los_azim_sat_ers[3];
     float sun_zen_surf_north[3];
     float sun_azim_surf_north[3];
     float los_zen_surf_north[3];
     float los_azim_surf_north[3];
     float lon[NUM_COORDS];
     float lat[NUM_COORDS];
};

struct glr2_gome
{
     int   pixel_nr;
     int   subsetcounter;
     unsigned int utc_date;
     unsigned int utc_time;
     float sat_geo_height;
     float earth_radius;
     float sat_zenith[3];
     float sat_sight[3];
     float sat_azim[3];
     float toa_zenith[3];
     float toa_sight[3];
     float toa_azim[3];
     float lon[NUM_COORDS];
     float lat[NUM_COORDS];
};

struct cr1_gome
{
     short  mode;
     short  type;
     float  surfaceHeight;
     float  fraction;
     float  fractionError;
     float  albedo;
     float  albedoError;
     float  height;
     float  heightError;
     float  thickness;
     float  thicknessError;
     float  topPress;
     float  topPressError;
};

union quality_fcd
{
     struct fcd_breakout
     {
	  unsigned int array_1:2;
	  unsigned int array_2:2;
	  unsigned int array_3:2;
	  unsigned int array_4:2;
	  unsigned int pmd_1:1;
	  unsigned int pmd_2:1;
	  unsigned int pmd_3:1;
	  unsigned int spare:5;
     } flag_fields;

     short flags;
};

struct lv1_bcr
{
     short array_nr;
     short start;
     short end;
};

struct lv1_kde
{
     float bsdf_1[SCIENCE_CHANNELS];
     float bsdf_2[SCIENCE_CHANNELS];
     float resp_1[SCIENCE_CHANNELS];
     float resp_2[SCIENCE_CHANNELS];
     float f2_1[SCIENCE_CHANNELS];
     float f2_2[SCIENCE_CHANNELS];
     float smdep_1[SCIENCE_CHANNELS];
     float smdep_2[SCIENCE_CHANNELS];
     float chi_1[SCIENCE_CHANNELS];
     float chi_2[SCIENCE_CHANNELS];
     float eta_1[SCIENCE_CHANNELS];
     float eta_2[SCIENCE_CHANNELS];
     float ksi_1[SCIENCE_CHANNELS];
     float ksi_2[SCIENCE_CHANNELS];
     float rfs[SCIENCE_PIXELS];
};

struct lv1_ghost
{
     short symmetry[SCIENCE_CHANNELS][NUM_STRAY_GHOSTS];
     short center[SCIENCE_CHANNELS][NUM_STRAY_GHOSTS];
     float defocus[SCIENCE_CHANNELS][NUM_STRAY_GHOSTS];
     float energy[SCIENCE_CHANNELS][NUM_STRAY_GHOSTS];
};

struct lv1_leak
{
     float noise;
     float pmd_offs[PMD_NUMBER];
     float pmd_noise;
     float dark[SCIENCE_PIXELS];
};

struct lv1_hot
{
     short record;
     short array;
     short pixel;
};

struct lv1_spec
{
     double coeffs[SCIENCE_CHANNELS][NUM_SPEC_COEFFS];
     double error[SCIENCE_CHANNELS];
};

struct lv1_calib
{
     float eta_omega[CHANNEL_SIZE];
     float response[CHANNEL_SIZE];
};

struct fcd_gome
{
     union quality_fcd detector_flags;
     short  npeltier;
     short  nleak;
     short  nhot;
     short  nspec;
     short  nang;
     short  width_conv;
     short  indx_spec;
     unsigned int sun_date;
     unsigned int sun_time;
     float  bsdf_0;
     float  elevation;
     float  azimuth;
     float  sun_pmd[PMD_NUMBER];
     float  sun_pmd_wv[PMD_NUMBER];
     float  stray_level[4];
     float  scale_peltier[NUM_FPA_SCALE];
     float  coeffs[8];
     float  filter_peltier[NUM_FPA_COEFFS];
     float  pixel_gain[SCIENCE_PIXELS];
     float  intensity[SCIENCE_PIXELS];
     float  sun_ref[SCIENCE_PIXELS];
     float  sun_precision[SCIENCE_PIXELS];
     struct lv1_ghost ghost;
     struct lv1_kde kde;
     struct lv1_bcr bcr[NUM_SPEC_BANDS];
     struct lv1_leak *leak;
     struct lv1_hot  *hot;
     struct lv1_spec *spec;
     struct lv1_calib *calib;
};

struct polar_gome
{
     float wv[NUM_POLAR_COEFFS];
     float coeff[NUM_POLAR_COEFFS];
     float error[NUM_POLAR_COEFFS];
     float chi;
};

struct mph0_gome
{
     char ProductConfidenceData[3];
     char UTC_MPH_Generation[25];
     char ProcessorSoftwareVersion[9];
};

struct sph0_gome
{
     char sph_5[2];
     char sph_6[21];
};


struct ihr_gome
{
     unsigned short subsetcounter;
     unsigned short prism_temp;
     unsigned short averagemode;
     union {
	  struct {
#ifdef _SWAP_TO_LITTLE_ENDIAN
	       unsigned char ch4:2;
	       unsigned char ch3:2;
	       unsigned char ch2b:2;
	       unsigned char ch2a:2;
	       unsigned char ch1b:2;
	       unsigned char ch1a:2;
	       unsigned char fpa2:1;
	       unsigned char fpa3:2;
	       unsigned char fpa4:1;
#else
	       unsigned char fpa4:1;
	       unsigned char fpa3:2;
	       unsigned char fpa2:1;
	       unsigned char ch1a:2;
	       unsigned char ch1b:2;
	       unsigned char ch2a:2;
	       unsigned char ch2b:2;
	       unsigned char ch3:2;
	       unsigned char ch4:2;
#endif	       
	  } stat;
	  unsigned short two_byte;
     } intg;
     unsigned short pmd[PMD_NUMBER][PMD_IN_GRID];
     short peltier[SCIENCE_CHANNELS];
};

struct pmd_gome
{
     struct glr1_gome glr;
     float value[PMD_NUMBER];
};

struct pcd_gome
{
     short selected;
     short indx_spec;
     short indx_leak;
     short indx_bands[NUM_SPEC_BANDS];
     float dark_current;
     float noise_factor;
     struct glr1_gome glr;
     struct cr1_gome  cld;
     struct polar_gome polar;
     struct mph0_gome mph0;
     struct sph0_gome sph0;
     struct ihr_gome  ihr;
     struct pmd_gome  pmd[PMD_IN_GRID];
};

struct smcd_gome
{
     short selected;
     short indx_spec;
     short indx_leak;
     short indx_bands[NUM_SPEC_BANDS];
     unsigned int utc_date;
     unsigned int utc_time;
     float north_sun_zen;
     float north_sun_azim;
     float north_sm_zen;
     float north_sm_azim;
     float sun_or_moon;
     float dark_current;
     float noise_factor;
     struct mph0_gome mph0;
     struct sph0_gome sph0;
     struct ihr_gome ihr;
     struct pmd_gome  pmd[PMD_IN_GRID];
};

union quality_rec
{
     struct rec_breakout
     {
	  unsigned int dead:2;
	  unsigned int hot:2;
	  unsigned int saturate:2;
	  unsigned int spectral:2;
     } flag_fields;

     short flags;
};

struct rec_gome
{
     union quality_rec pixel_flags;
     short indx_psp;
     short indx_pcd;
     float integration[2];
     float wave[CHANNEL_SIZE];
     float data[CHANNEL_SIZE];
};

struct irr1_gome
{
     short indx_vcd;
     short indx_doas;
     short indx_amf;
     short indx_icfa;
     short indx_stats;

     float cloud_frac;
     float cloud_pres;
     float err_cloud_frac;
     float err_cloud_pres;
     float surface_pres;
     float cca_cloud_frac;
     signed char cca_subpixel[16];

     float pmd_avg[3];
     float pmd_sdev[3];
     float pixel_color[16];
     float pixel_gradient;

     float total_vcd[LVL2_MAX_NMOL];
     float error_vcd[LVL2_MAX_NMOL];
     float slant_doas[LVL2_MAX_NMOL];
     float error_doas[LVL2_MAX_NMOL];
     float rms_doas[LVL2_MAX_NWIN];
     float chi_doas[LVL2_MAX_NWIN];
     float fit_doas[LVL2_MAX_NWIN];
     float iter_doas[LVL2_MAX_NWIN];
     float ground_amf[LVL2_MAX_NMOL];
     float cloud_amf[LVL2_MAX_NMOL];
     float intensity_ground[LVL2_MAX_NWIN];
     float intensity_cloud[LVL2_MAX_NWIN];
     float intensity_measured[LVL2_MAX_NWIN];
};

struct irr2_gome
{
     short indx_vcd;
     short indx_doas;
     short indx_amf;

     float ozone_temperature;
     float ozone_ring_corr;
     float ghost_column;
     float cld_frac;
     float error_cld_frac;
     float cld_height;
     float error_cld_height;
     float cld_press;
     float error_cld_press;
     float cld_albedo;
     float error_cld_albedo;
     float surface_height;
     float surface_press;
     float surface_albedo;

     float total_vcd[LVL2_V2_NMOL];
     float error_vcd[LVL2_V2_NMOL];
     float slant_doas[LVL2_V2_NMOL];
     float error_doas[LVL2_V2_NMOL];
     float rms_doas[LVL2_V2_NWIN];
     float chi_doas[LVL2_V2_NWIN];
     float fit_doas[LVL2_V2_NWIN];
     float iter_doas[LVL2_V2_NWIN];
     float ground_amf[LVL2_V2_NMOL];
     float error_ground_amf[LVL2_V2_NMOL];
     float cloud_amf[LVL2_V2_NMOL];
     float error_cloud_amf[LVL2_V2_NMOL];
};

struct ddr_gome
{
     struct glr2_gome glr;
     struct irr1_gome *irr1;
     struct irr2_gome *irr2;
     float ozone;
     float error;
};

#ifdef _WITH_SQL
#endif

#ifdef __cplusplus
  }
#endif
#endif /* __DEFS_GOME */
