/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.COPYRIGHT (c) 2008 - 2013 SRON (R.M.van.Hees@sron.nl)

   This is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License, version 2, as
   Free Software Foundation; either version 2.

   The software is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA  02111-1307, USA.

.IDENTifer   MERIS_VERSION
.AUTHOR      R.M. van Hees
.KEYWORDS    NADC software
.LANGUAGE    ANSI C
.PURPOSE     Returns the library version numbers through arguments.
.COMMENTS    contains MERISget_Version and MERISshow_Version
.ENVIRONment none
.VERSION     1.0     22-Sep-2008   Created by R. M. van Hees
------------------------------------------------------------*/
/*
 * Define _POSIX_SOURCE to indicate
 * that this is a POSIX program
 */
#define  _POSIX_C_SOURCE 2

/*+++++ System headers +++++*/
#include <stdio.h>
#include <string.h>

/*+++++ Local Headers +++++*/
#define _MERIS_COMMON
#include <defs_nadc.h>
#include <version.h>
#include "../VERSION"

/*+++++++++++++++++++++++++ Main Program or Function +++++++++++++++*/
/*+++++++++++++++++++++++++
.IDENTifer   MERISget_Version
.PURPOSE     get version of MERIS library and release date
.INPUT/OUTPUT
  call as    MERISget_Version( &majnum, &minnum, &relnum, reldate );
    output:
             unsigned int *majnum :  major revision number
	     unsigned int *minnum :  minor revision number
	     unsigned int *relnum :  release revision number
             char *reldate        :  release date

.RETURNS     nothing
.COMMENTS    none
-------------------------*/
void MERISget_Version( unsigned int *majnum, unsigned int *minnum, 
		      unsigned int *relnum, char *reldate )
{
     if ( majnum ) *majnum = MER_VERS_MAJOR;
     if ( minnum ) *minnum = MER_VERS_MINOR;
     if ( relnum ) *relnum = MER_VERS_RELEASE;
     if ( reldate ) 
	  (void) strlcpy( reldate, MER_VERS_DATE, MAX_STRING_LENGTH );
}

/*+++++++++++++++++++++++++
.IDENTifer   MERISshow_Version
.PURPOSE     display version of MERIS library and release date
.INPUT/OUTPUT
  call as    MERISshow_Version( stream, prognm );
    output:
             FILE *stream  :   file pointer
	     char *prognm  :   name of the calling program

.RETURNS     nothing
.COMMENTS    none
-------------------------*/
void MERISshow_Version( FILE *stream, const char prognm[] )
{
     unsigned int majnum, minnum, relnum;
     char reldate[MAX_STRING_LENGTH];

     (void) fprintf( stream, "%s (version %s.%s) is linked with:\n", 
		     prognm, major_version, minor_version );
     MERISget_Version( &majnum, &minnum, &relnum, reldate );
     (void) fprintf( stream,
		     "\tlibMERIS version %-u.%-u release %-u (%s)\n", 
		     majnum, minnum, relnum, reldate );
     NADCget_Version( &majnum, &minnum, &relnum, reldate );
     (void) fprintf( stream,
		     "\tlibNADC version %-u.%-u release %-u (%s)\n", 
		     majnum, minnum, relnum, reldate );
     NADC_CopyRight( stream );
}
